/* Library for Arduino Uno board and compatible devices. (probably all the Arduino boards)
 *
 * This library contains several functions for using with a DDS generator
 * AD9850 module. All the functions are intended to work with this device
 * in SERIAL mode, so you need to connect the parallel pins D0 and D1 to VCC
 * and D2 GND, as written in the AD9850 datasheet. For further information
 * check the Analog Devices website.
 *
 * Coded by
 *   Gonzalo Recio - http://grsynth.com
 * Examples and 1.6.x compatibility
 *   Robert Ulbricht  
 */

#include "Arduino.h"
#include "ARDUINO_AD9850.h"
#define _DEBUG 1

//Constructor for the class AD9850
AD9850::AD9850(int givenW_CLK, int givenFQ_UD, int givenDATA, int givenRESET)
{
W_CLK = givenW_CLK;
FQ_UD = givenFQ_UD;
DATA  = givenDATA;
RESET = givenRESET;

pinMode(W_CLK, OUTPUT);
pinMode(FQ_UD, OUTPUT);
pinMode(DATA, OUTPUT);
pinMode(RESET, OUTPUT);
}

void AD9850::init()
{
digitalWrite(W_CLK, LOW);
digitalWrite(FQ_UD, LOW);
digitalWrite(DATA, LOW);
digitalWrite(RESET, LOW);
#if _DEBUG
Serial.println("All the pins lowered.");
#endif
}

/* Reset operation for the AD9850. This function must be called before using AD9850_Osc
 * first time in the execution (check page 12 on datasheet)
 */
void AD9850::doReset()
{
digitalWrite(W_CLK, LOW);
digitalWrite(W_CLK, HIGH);
digitalWrite(W_CLK, LOW);

digitalWrite(RESET, LOW);
digitalWrite(RESET, HIGH);
digitalWrite(RESET, LOW);

digitalWrite(FQ_UD, LOW);
digitalWrite(FQ_UD, HIGH);
digitalWrite(FQ_UD, LOW);

osc(0,0);

#if _DEBUG
Serial.println("Reset succesfull.");
#endif
}

/* Sets the DDS sine and square oscillator to the detailed "frequency" and "phase" variables.
 * "frequency" will be turned into a 32 bit word, so the frequency resolution of 0.0291 Hz
 * with a 125 MHz reference clock. "phase" will be a 5 bit word instead so the resolution is
 * 11.5 degrees, or pi/32 radians.
 */
void AD9850::osc(double frequency, double phase)
{
#if _DEBUG
Serial.print("Changing frequency to: ");
Serial.println(frequency);
#endif

long y=(long)frequency*FREQ_FACTOR/XTAL_MHZ;
while(phase>360)
	phase-=360;
long z=phase/11.5;
        
#if _DEBUG
Serial.print("Frequency word: ");
Serial.println(y);
#endif
	
int i;
        
//Frequency 32-bit word
for (i=0;i<32;i++){
  digitalWrite(DATA, (y>>i) & 0x01);
	digitalWrite(W_CLK, HIGH);
	digitalWrite(W_CLK, LOW);
	}
	
//control bit #1, control bit #2 and Power off, all to low
digitalWrite(DATA, LOW);
digitalWrite(W_CLK, HIGH);
digitalWrite(W_CLK, LOW);
digitalWrite(W_CLK, HIGH);
digitalWrite(W_CLK, LOW);
digitalWrite(W_CLK, HIGH);
digitalWrite(W_CLK, LOW);
	
//phase 5-bit word
for (i=0;i<5;i++){
  digitalWrite(DATA, (z>>i) & 0x01);
	digitalWrite(W_CLK, HIGH);
	digitalWrite(W_CLK, LOW);
	}

digitalWrite(FQ_UD, HIGH);
digitalWrite(FQ_UD, LOW);
}

/* Enables power down mode. This method is used for a quick "all outputs" disable.
 * The effect is the same as AD9850_Osc(0,0), but it takes less clock cycles
 */
void AD9850::powerDown()
{
#if _DEBUG
Serial.println("Changing mode to power down.");
#endif
int PDword=0x04;
int i;
digitalWrite(FQ_UD, HIGH);
digitalWrite(FQ_UD, LOW);

for (i=0;i<8;i++){
	if((PDword>>i)&0x01)
		digitalWrite(DATA, HIGH);
	else
		digitalWrite(DATA, LOW);
	digitalWrite(W_CLK, HIGH);
	digitalWrite(W_CLK, LOW);
	}

digitalWrite(FQ_UD, HIGH);
digitalWrite(FQ_UD, LOW);
}

/* Performs a frequency sweep increased in "inc"Hz steps. The frequency order is from low to high
 * and resets to the lower frequency when maximum frequency is reached.
 */
void AD9850::sweepUp(double minFreq, double maxFreq, double inc, int microsPerDelay)
{
double f=minFreq;
int i;
while (1){
  osc(f,0);
  f+=inc;
  if (f>maxFreq)
    f=minFreq;
  delayMicroseconds(microsPerDelay);
  }
}

/* Performs a frequency sweep decreased in "inc"Hz steps. The frequency order is from high to low
 * and resets to the higher frequency when minimum frequency is reached.
 */
void AD9850::sweepDown(double minFreq, double maxFreq, double inc, int microsPerDelay)
{
double f=maxFreq;
int i;
while (1){
  osc(f,0);
  f-=inc;
  if (f<minFreq)
    f=maxFreq;
  delayMicroseconds(microsPerDelay);
  }
}

/* Performs a frequency sweep increased in "inc"Hz steps. The frequency order is from low to high,
 * but it changes to from high to low when the maximum frequency is reached and so on.
 */
void AD9850::sweepLoop(double minFreq, double maxFreq, double inc, int microsPerDelay)
{
double f=minFreq;
int i;
while (1){
  osc(f,0);
  f+=inc;
  if (f>maxFreq || f<minFreq)
    inc=-inc;
  delayMicroseconds(microsPerDelay);
  }
}
